#version 430 core

// Task 2: When TASK_TWO_METHOD == TASK_TWO_METHOD_USE, you require the following extension, so uncomment the code
//#extension GL_ARB_shader_draw_parameters : require

// Input variables
layout (location = 0) in vec4 position;
layout (location = 1) in vec3 normal;

// Output variables
out VertexData
{
	vec3 normal_ws;		// Normal in world space
	vec3 position_ws;	// Position in world space
	flat int object_idx;
} outData;

// Data of the camera
layout (std140, binding = 0) uniform CameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};

// Data of the object
struct SingleModelData
{
	mat4 model;			// Model matrix
	mat4 model_inv;		// Inverse of the model matrix
	mat3 model_it;		// Inverse of the transpose of the top-left part 3x3 of the model matrix
};
layout (std140, binding = 2) buffer ModelData
{
	SingleModelData models[];
};

uniform int model_idx;

//-----------------------------------------------------------------------

void main()
{
	// Task 2: When TASK_TWO_METHOD == TASK_TWO_METHOD_USE, object_idx is in gl_DrawIDARB
	//			instead of model_idx, so comment/uncomment the following lines
	int object_idx = model_idx;
	//int object_idx = gl_DrawIDARB;

	// Compute the data of the vertex
	outData.position_ws = vec3(models[object_idx].model * position);
	outData.normal_ws = normalize(models[object_idx].model_it * normal);
	outData.object_idx = object_idx;

	// Compute the gl_Position
	gl_Position = projection * view * models[object_idx].model * position;
}
